@@ -1541,6 +1541,25 @@ pub enum MarkerTree {
     Or(Vec<MarkerTree>),
 }
 
+impl<'de> Deserialize<'de> for MarkerTree {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let s = String::deserialize(deserializer)?;
+        FromStr::from_str(&s).map_err(de::Error::custom)
+    }
+}
+
+impl Serialize for MarkerTree {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_str(&self.to_string())
+    }
+}
+
 impl FromStr for MarkerTree {
     type Err = Pep508Error;
 
@@ -1601,17 +1620,42 @@ impl MarkerTree {
     /// For example, if `dev` is a provided extra, given `sys_platform == 'linux' and extra == 'dev'`,
     /// the marker will be simplified to `sys_platform == 'linux'`.
     pub fn simplify_extras(self, extras: &[ExtraName]) -> Option<MarkerTree> {
+        self.simplify_extras_with(|name| extras.contains(name))
+    }
+
+    /// Remove the extras from a marker, returning `None` if the marker tree evaluates to `true`.
+    ///
+    /// Any `extra` markers that are always `true` given the provided predicate will be removed.
+    /// Any `extra` markers that are always `false` given the provided predicate will be left
+    /// unchanged.
+    ///
+    /// For example, if `is_extra('dev')` is true, given
+    /// `sys_platform == 'linux' and extra == 'dev'`, the marker will be simplified to
+    /// `sys_platform == 'linux'`.
+    pub fn simplify_extras_with(self, is_extra: impl Fn(&ExtraName) -> bool) -> Option<MarkerTree> {
+        // Because `simplify_extras_with_impl` is recursive, and we need to use
+        // our predicate in recursive calls, we need the predicate itself to
+        // have some indirection (or else we'd have to clone it). To avoid a
+        // recursive type at codegen time, we just introduce the indirection
+        // here, but keep the calling API ergonomic.
+        self.simplify_extras_with_impl(&is_extra)
+    }
+
+    fn simplify_extras_with_impl(
+        self,
+        is_extra: &impl Fn(&ExtraName) -> bool,
+    ) -> Option<MarkerTree> {
         /// Returns `true` if the given expression is always `true` given the set of extras.
-        fn is_true(expression: &MarkerExpression, extras: &[ExtraName]) -> bool {
+        fn is_true(expression: &MarkerExpression, is_extra: impl Fn(&ExtraName) -> bool) -> bool {
             match expression {
                 MarkerExpression::Extra {
                     operator: ExtraOperator::Equal,
                     name,
-                } => extras.contains(name),
+                } => is_extra(name),
                 MarkerExpression::Extra {
                     operator: ExtraOperator::NotEqual,
                     name,
-                } => !extras.contains(name),
+                } => !is_extra(name),
                 _ => false,
             }
         }
@@ -1619,7 +1663,7 @@ impl MarkerTree {
         match self {
             Self::Expression(expression) => {
                 // If the expression is true, we can remove the marker entirely.
-                if is_true(&expression, extras) {
+                if is_true(&expression, is_extra) {
                     None
                 } else {
                     // If not, return the original marker.
@@ -1630,7 +1674,7 @@ impl MarkerTree {
                 // Remove any expressions that are _true_ due to the presence of an extra.
                 let simplified = expressions
                     .into_iter()
-                    .filter_map(|marker| marker.simplify_extras(extras))
+                    .filter_map(|marker| marker.simplify_extras_with_impl(is_extra))
                     .collect::<Vec<_>>();
 
                 // If there are no expressions left, return None.
@@ -1650,7 +1694,7 @@ impl MarkerTree {
                 // Remove any expressions that are _true_ due to the presence of an extra.
                 let simplified = expressions
                     .into_iter()
-                    .filter_map(|marker| marker.simplify_extras(extras))
+                    .filter_map(|marker| marker.simplify_extras_with_impl(is_extra))
                     .collect::<Vec<_>>();
 
                 // If _any_ of the expressions are true (i.e., if any of the markers were filtered
@@ -1801,6 +1845,66 @@ impl MarkerTree {
             }
         }
     }
+
+    /// Combine this marker tree with the one given via a conjunction.
+    ///
+    /// This does some shallow flattening. That is, if `self` is a conjunction
+    /// already, then `tree` is added to it instead of creating a new
+    /// conjunction.
+    pub fn and(&mut self, tree: MarkerTree) {
+        match *self {
+            MarkerTree::Expression(_) | MarkerTree::Or(_) => {
+                let this = std::mem::replace(self, MarkerTree::And(vec![]));
+                *self = MarkerTree::And(vec![this]);
+            }
+            _ => {}
+        }
+        if let MarkerTree::And(ref mut exprs) = *self {
+            exprs.push(tree);
+        }
+    }
+
+    /// Combine this marker tree with the one given via a disjunction.
+    ///
+    /// This does some shallow flattening. That is, if `self` is a disjunction
+    /// already, then `tree` is added to it instead of creating a new
+    /// disjunction.
+    pub fn or(&mut self, tree: MarkerTree) {
+        match *self {
+            MarkerTree::Expression(_) | MarkerTree::And(_) => {
+                let this = std::mem::replace(self, MarkerTree::And(vec![]));
+                *self = MarkerTree::Or(vec![this]);
+            }
+            _ => {}
+        }
+        if let MarkerTree::Or(ref mut exprs) = *self {
+            exprs.push(tree);
+        }
+    }
+
+    /// Normalizes this marker tree such that all conjunctions and disjunctions
+    /// are sorted.
+    ///
+    /// This is useful in cases where creating conjunctions or disjunctions
+    /// might occur in a non-deterministic order. This routine will erase the
+    /// distinction created by such a construction.
+    pub fn normalize(&mut self) {
+        match *self {
+            MarkerTree::Expression(_) => {}
+            MarkerTree::And(ref mut trees) | MarkerTree::Or(ref mut trees) => {
+                // This is kind of cheesy, because we're doing a recursive call
+                // followed by a sort, and that sort is also recursive (due to
+                // the corresponding Ord impl being recursive).
+                //
+                // We should consider refactoring `MarkerTree` to a "smart
+                // constructor" design that normalizes them by construction.
+                for tree in &mut *trees {
+                    tree.normalize();
+                }
+                trees.sort();
+            }
+        }
+    }
 }
 
 impl Display for MarkerTree {
