@@ -10,10 +10,10 @@ use tracing::{debug, trace};
 use pep508_rs::VerbatimUrl;
 use pypi_types::{Requirement, RequirementSource};
 use uv_fs::{absolutize_path, Simplified};
-use uv_normalize::{ExtraName, PackageName};
+use uv_normalize::PackageName;
 use uv_warnings::warn_user;
 
-use crate::pyproject::{PyProjectToml, Source, ToolUvWorkspace};
+use crate::pyproject::{Project, PyProjectToml, Source, ToolUvWorkspace};
 
 #[derive(thiserror::Error, Debug)]
 pub enum WorkspaceError {
@@ -118,7 +118,11 @@ impl Workspace {
         let current_project = pyproject_toml
             .project
             .clone()
-            .map(|project| (project.name.clone(), project_path, pyproject_toml));
+            .map(|project| WorkspaceMember {
+                root: project_path,
+                project,
+                pyproject_toml,
+            });
         Self::collect_members(
             workspace_root,
             workspace_definition,
@@ -134,21 +138,9 @@ impl Workspace {
     /// Returns `None` if the package is not part of the workspace.
     pub fn with_current_project(self, package_name: PackageName) -> Option<ProjectWorkspace> {
         let member = self.packages.get(&package_name)?;
-        let extras = member
-            .pyproject_toml
-            .project
-            .as_ref()
-            .and_then(|project| project.optional_dependencies.as_ref())
-            .map(|optional_dependencies| {
-                let mut extras = optional_dependencies.keys().cloned().collect::<Vec<_>>();
-                extras.sort_unstable();
-                extras
-            })
-            .unwrap_or_default();
         Some(ProjectWorkspace {
             project_root: member.root().clone(),
             project_name: package_name,
-            extras,
             workspace: self,
         })
     }
@@ -174,7 +166,7 @@ impl Workspace {
         workspace_root: PathBuf,
         workspace_definition: ToolUvWorkspace,
         workspace_pyproject_toml: PyProjectToml,
-        current_project: Option<(PackageName, PathBuf, PyProjectToml)>,
+        current_project: Option<WorkspaceMember>,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Workspace, WorkspaceError> {
         let mut workspace_members = BTreeMap::new();
@@ -185,7 +177,7 @@ impl Workspace {
         // project.
         if current_project
             .as_ref()
-            .map(|(_, path, _)| path != &workspace_root)
+            .map(|root_member| root_member.root != workspace_root)
             .unwrap_or(true)
         {
             if let Some(project) = &workspace_pyproject_toml.project {
@@ -204,6 +196,7 @@ impl Workspace {
                     project.name.clone(),
                     WorkspaceMember {
                         root: workspace_root.clone(),
+                        project: project.clone(),
                         pyproject_toml,
                     },
                 );
@@ -211,20 +204,14 @@ impl Workspace {
         }
 
         // The current project is a workspace member, especially in a single project workspace.
-        if let Some((project_name, project_path, project)) = current_project {
+        if let Some(root_member) = current_project {
             debug!(
                 "Adding current workspace member: {}",
-                project_path.simplified_display()
+                root_member.root.simplified_display()
             );
 
-            seen.insert(project_path.clone());
-            workspace_members.insert(
-                project_name,
-                WorkspaceMember {
-                    root: project_path.clone(),
-                    pyproject_toml: project.clone(),
-                },
-            );
+            seen.insert(root_member.root.clone());
+            workspace_members.insert(root_member.project.name.clone(), root_member);
         }
 
         // Add all other workspace members.
@@ -259,16 +246,18 @@ impl Workspace {
                     return Err(WorkspaceError::MissingProject(member_root));
                 };
 
-                let member = WorkspaceMember {
-                    root: member_root.clone(),
-                    pyproject_toml,
-                };
-
                 debug!(
                     "Adding discovered workspace member: {}",
                     member_root.simplified_display()
                 );
-                workspace_members.insert(project.name, member);
+                workspace_members.insert(
+                    project.name.clone(),
+                    WorkspaceMember {
+                        root: member_root.clone(),
+                        project,
+                        pyproject_toml,
+                    },
+                );
             }
         }
         let workspace_sources = workspace_pyproject_toml
@@ -293,6 +282,9 @@ impl Workspace {
 pub struct WorkspaceMember {
     /// The path to the project root.
     root: PathBuf,
+    /// The `[project]` table, from the `pyproject.toml` of the project found at
+    /// `<root>/pyproject.toml`.
+    project: Project,
     /// The `pyproject.toml` of the project, found at `<root>/pyproject.toml`.
     pyproject_toml: PyProjectToml,
 }
@@ -303,6 +295,12 @@ impl WorkspaceMember {
         &self.root
     }
 
+    /// The `[project]` table, from the `pyproject.toml` of the project found at
+    /// `<root>/pyproject.toml`.
+    pub fn project(&self) -> &Project {
+        &self.project
+    }
+
     /// The `pyproject.toml` of the project, found at `<root>/pyproject.toml`.
     pub fn pyproject_toml(&self) -> &PyProjectToml {
         &self.pyproject_toml
@@ -393,8 +391,6 @@ pub struct ProjectWorkspace {
     project_root: PathBuf,
     /// The name of the package.
     project_name: PackageName,
-    /// The extras available in the project.
-    extras: Vec<ExtraName>,
     /// The workspace the project is part of.
     workspace: Workspace,
 }
@@ -444,13 +440,7 @@ impl ProjectWorkspace {
             .clone()
             .ok_or_else(|| WorkspaceError::MissingProject(pyproject_path.clone()))?;
 
-        Self::from_project(
-            project_root,
-            &pyproject_toml,
-            project.name,
-            stop_discovery_at,
-        )
-        .await
+        Self::from_project(project_root, &project, &pyproject_toml, stop_discovery_at).await
     }
 
     /// If the current directory contains a `pyproject.toml` with a `project` table, discover the
@@ -475,13 +465,7 @@ impl ProjectWorkspace {
         };
 
         Ok(Some(
-            Self::from_project(
-                project_root,
-                &pyproject_toml,
-                project.name,
-                stop_discovery_at,
-            )
-            .await?,
+            Self::from_project(project_root, &project, &pyproject_toml, stop_discovery_at).await?,
         ))
     }
 
@@ -496,11 +480,6 @@ impl ProjectWorkspace {
         &self.project_name
     }
 
-    /// Returns the extras available in the project.
-    pub fn project_extras(&self) -> &[ExtraName] {
-        &self.extras
-    }
-
     /// Returns the [`Workspace`] containing the current project.
     pub fn workspace(&self) -> &Workspace {
         &self.workspace
@@ -516,7 +495,15 @@ impl ProjectWorkspace {
     pub fn requirements(&self) -> Vec<Requirement> {
         vec![Requirement {
             name: self.project_name.clone(),
-            extras: self.extras.clone(),
+            extras: self.workspace().packages[&self.project_name]
+                .pyproject_toml
+                .project
+                .as_ref()
+                .and_then(|project| project.optional_dependencies.as_ref())
+                .map(|optional_dependencies| {
+                    optional_dependencies.keys().cloned().collect::<Vec<_>>()
+                })
+                .unwrap_or_default(),
             marker: None,
             source: RequirementSource::Path {
                 path: self.project_root.clone(),
@@ -530,33 +517,27 @@ impl ProjectWorkspace {
     /// Find the workspace for a project.
     pub async fn from_project(
         project_path: &Path,
-        project: &PyProjectToml,
-        project_name: PackageName,
+        project: &Project,
+        project_pyproject_toml: &PyProjectToml,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Self, WorkspaceError> {
         let project_path = absolutize_path(project_path)
             .map_err(WorkspaceError::Normalize)?
             .to_path_buf();
 
-        // Extract the extras available in the project.
-        let extras = project
-            .project
-            .as_ref()
-            .and_then(|project| project.optional_dependencies.as_ref())
-            .map(|optional_dependencies| {
-                let mut extras = optional_dependencies.keys().cloned().collect::<Vec<_>>();
-                extras.sort_unstable();
-                extras
-            })
-            .unwrap_or_default();
-
         // Check if the current project is also an explicit workspace root.
-        let mut workspace = project
+        let mut workspace = project_pyproject_toml
             .tool
             .as_ref()
             .and_then(|tool| tool.uv.as_ref())
             .and_then(|uv| uv.workspace.as_ref())
-            .map(|workspace| (project_path.clone(), workspace.clone(), project.clone()));
+            .map(|workspace| {
+                (
+                    project_path.clone(),
+                    workspace.clone(),
+                    project_pyproject_toml.clone(),
+                )
+            });
 
         if workspace.is_none() {
             // The project isn't an explicit workspace root, check if we're a regular workspace
@@ -564,22 +545,23 @@ impl ProjectWorkspace {
             workspace = find_workspace(&project_path, stop_discovery_at).await?;
         }
 
+        let current_project = WorkspaceMember {
+            root: project_path.clone(),
+            project: project.clone(),
+            pyproject_toml: project_pyproject_toml.clone(),
+        };
+
         let Some((workspace_root, workspace_definition, workspace_pyproject_toml)) = workspace
         else {
             // The project isn't an explicit workspace root, but there's also no workspace root
             // above it, so the project is an implicit workspace root identical to the project root.
             debug!("No workspace root found, using project root");
-            let current_project_as_members = BTreeMap::from_iter([(
-                project_name.clone(),
-                WorkspaceMember {
-                    root: project_path.clone(),
-                    pyproject_toml: project.clone(),
-                },
-            )]);
+
+            let current_project_as_members =
+                BTreeMap::from_iter([(project.name.clone(), current_project)]);
             return Ok(Self {
                 project_root: project_path.clone(),
-                project_name: project_name.clone(),
-                extras,
+                project_name: project.name.clone(),
                 workspace: Workspace {
                     root: project_path.clone(),
                     packages: current_project_as_members,
@@ -599,44 +581,17 @@ impl ProjectWorkspace {
             workspace_root,
             workspace_definition,
             workspace_pyproject_toml,
-            Some((project_name.clone(), project_path.clone(), project.clone())),
+            Some(current_project),
             stop_discovery_at,
         )
         .await?;
 
         Ok(Self {
             project_root: project_path,
-            project_name,
-            extras,
+            project_name: project.name.clone(),
             workspace,
         })
     }
-
-    /// Used in tests.
-    pub fn dummy(root: &Path, project_name: &PackageName) -> Self {
-        // This doesn't necessarily match the exact test case, but we don't use the other fields
-        // for the test cases atm.
-        let root_member = WorkspaceMember {
-            root: root.to_path_buf(),
-            pyproject_toml: PyProjectToml {
-                project: Some(crate::pyproject::Project {
-                    name: project_name.clone(),
-                    optional_dependencies: None,
-                }),
-                tool: None,
-            },
-        };
-        Self {
-            project_root: root.to_path_buf(),
-            project_name: project_name.clone(),
-            extras: Vec::new(),
-            workspace: Workspace {
-                root: root.to_path_buf(),
-                packages: [(project_name.clone(), root_member)].into_iter().collect(),
-                sources: BTreeMap::default(),
-            },
-        }
-    }
 }
 
 /// Find the workspace root above the current project, if any.
@@ -864,12 +819,16 @@ mod tests {
         {
           "project_root": "[ROOT]/albatross-in-example/examples/bird-feeder",
           "project_name": "bird-feeder",
-          "extras": [],
           "workspace": {
             "root": "[ROOT]/albatross-in-example/examples/bird-feeder",
             "packages": {
               "bird-feeder": {
                 "root": "[ROOT]/albatross-in-example/examples/bird-feeder",
+                "project": {
+                  "name": "bird-feeder",
+                  "requires-python": ">=3.12",
+                  "optional-dependencies": null
+                },
                 "pyproject_toml": "[PYPROJECT_TOML]"
               }
             },
@@ -895,12 +854,16 @@ mod tests {
             {
               "project_root": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
               "project_name": "bird-feeder",
-              "extras": [],
               "workspace": {
                 "root": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
                 "packages": {
                   "bird-feeder": {
                     "root": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
+                    "project": {
+                      "name": "bird-feeder",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   }
                 },
@@ -925,20 +888,34 @@ mod tests {
             {
               "project_root": "[ROOT]/albatross-root-workspace",
               "project_name": "albatross",
-              "extras": [],
               "workspace": {
                 "root": "[ROOT]/albatross-root-workspace",
                 "packages": {
                   "albatross": {
                     "root": "[ROOT]/albatross-root-workspace",
+                    "project": {
+                      "name": "albatross",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   },
                   "bird-feeder": {
                     "root": "[ROOT]/albatross-root-workspace/packages/bird-feeder",
+                    "project": {
+                      "name": "bird-feeder",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   },
                   "seeds": {
                     "root": "[ROOT]/albatross-root-workspace/packages/seeds",
+                    "project": {
+                      "name": "seeds",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   }
                 },
@@ -969,20 +946,34 @@ mod tests {
             {
               "project_root": "[ROOT]/albatross-virtual-workspace/packages/albatross",
               "project_name": "albatross",
-              "extras": [],
               "workspace": {
                 "root": "[ROOT]/albatross-virtual-workspace",
                 "packages": {
                   "albatross": {
                     "root": "[ROOT]/albatross-virtual-workspace/packages/albatross",
+                    "project": {
+                      "name": "albatross",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   },
                   "bird-feeder": {
                     "root": "[ROOT]/albatross-virtual-workspace/packages/bird-feeder",
+                    "project": {
+                      "name": "bird-feeder",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   },
                   "seeds": {
                     "root": "[ROOT]/albatross-virtual-workspace/packages/seeds",
+                    "project": {
+                      "name": "seeds",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   }
                 },
@@ -1007,12 +998,16 @@ mod tests {
             {
               "project_root": "[ROOT]/albatross-just-project",
               "project_name": "albatross",
-              "extras": [],
               "workspace": {
                 "root": "[ROOT]/albatross-just-project",
                 "packages": {
                   "albatross": {
                     "root": "[ROOT]/albatross-just-project",
+                    "project": {
+                      "name": "albatross",
+                      "requires-python": ">=3.12",
+                      "optional-dependencies": null
+                    },
                     "pyproject_toml": "[PYPROJECT_TOML]"
                   }
                 },
