@@ -2,13 +2,14 @@ use itertools::Either;
 use std::env;
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
+use uv_configuration::PreviewMode;
 
 use same_file::is_same_file;
 
 use uv_cache::Cache;
 use uv_fs::{LockedFile, Simplified};
 
-use crate::discovery::{InterpreterRequest, SourceSelector, SystemPython, VersionRequest};
+use crate::discovery::{InterpreterRequest, SourceSelector, SystemPython};
 use crate::virtualenv::{virtualenv_python_executable, PyVenvConfiguration};
 use crate::{
     find_default_interpreter, find_interpreter, Error, Interpreter, InterpreterSource, Target,
@@ -25,25 +26,37 @@ struct PythonEnvironmentShared {
 }
 
 impl PythonEnvironment {
-    /// Create a [`PythonEnvironment`] from a user request.
-    pub fn find(python: Option<&str>, system: SystemPython, cache: &Cache) -> Result<Self, Error> {
+    /// Find a [`PythonEnvironment`].
+    ///
+    /// This is the standard interface for discovering a Python environment for use with uv.
+    pub fn find(
+        python: Option<&str>,
+        system: SystemPython,
+        preview: PreviewMode,
+        cache: &Cache,
+    ) -> Result<Self, Error> {
         // Detect the current Python interpreter.
         if let Some(python) = python {
-            Self::from_requested_python(python, system, cache)
+            Self::from_requested_python(python, system, preview, cache)
         } else if system.is_preferred() {
-            Self::from_default_python(cache)
+            Self::from_default_python(preview, cache)
         } else {
             // First check for a parent intepreter
-            match Self::from_parent_interpreter(system, cache) {
-                Ok(env) => return Ok(env),
-                Err(Error::NotFound(_)) => {}
-                Err(err) => return Err(err),
+            // We gate this check to avoid an extra log message when it is not set
+            if std::env::var_os("UV_INTERNAL__PARENT_INTERPRETER").is_some() {
+                match Self::from_parent_interpreter(system, cache) {
+                    Ok(env) => return Ok(env),
+                    Err(Error::NotFound(_)) => {}
+                    Err(err) => return Err(err),
+                }
             }
 
             // Then a virtual environment
             match Self::from_virtualenv(cache) {
                 Ok(venv) => Ok(venv),
-                Err(Error::NotFound(_)) if system.is_allowed() => Self::from_default_python(cache),
+                Err(Error::NotFound(_)) if system.is_allowed() => {
+                    Self::from_default_python(preview, cache)
+                }
                 Err(err) => Err(err),
             }
         }
@@ -54,7 +67,7 @@ impl PythonEnvironment {
     /// Allows Conda environments (via `CONDA_PREFIX`) though they are not technically virtual environments.
     pub fn from_virtualenv(cache: &Cache) -> Result<Self, Error> {
         let sources = SourceSelector::VirtualEnv;
-        let request = InterpreterRequest::Version(VersionRequest::Default);
+        let request = InterpreterRequest::Any;
         let found = find_interpreter(&request, SystemPython::Disallowed, &sources, cache)??;
 
         debug_assert!(
@@ -74,7 +87,7 @@ impl PythonEnvironment {
     /// Create a [`PythonEnvironment`] for the parent interpreter i.e. the executable in `python -m uv ...`
     pub fn from_parent_interpreter(system: SystemPython, cache: &Cache) -> Result<Self, Error> {
         let sources = SourceSelector::from_sources([InterpreterSource::ParentInterpreter]);
-        let request = InterpreterRequest::Version(VersionRequest::Default);
+        let request = InterpreterRequest::Any;
         let found = find_interpreter(&request, system, &sources, cache)??;
 
         Ok(Self(Arc::new(PythonEnvironmentShared {
@@ -107,9 +120,10 @@ impl PythonEnvironment {
     pub fn from_requested_python(
         request: &str,
         system: SystemPython,
+        preview: PreviewMode,
         cache: &Cache,
     ) -> Result<Self, Error> {
-        let sources = SourceSelector::from_settings(system);
+        let sources = SourceSelector::from_settings(system, preview);
         let request = InterpreterRequest::parse(request);
         let interpreter = find_interpreter(&request, system, &sources, cache)??.into_interpreter();
         Ok(Self(Arc::new(PythonEnvironmentShared {
@@ -119,8 +133,8 @@ impl PythonEnvironment {
     }
 
     /// Create a [`PythonEnvironment`] for the default Python interpreter.
-    pub fn from_default_python(cache: &Cache) -> Result<Self, Error> {
-        let interpreter = find_default_interpreter(cache)??.into_interpreter();
+    pub fn from_default_python(preview: PreviewMode, cache: &Cache) -> Result<Self, Error> {
+        let interpreter = find_default_interpreter(preview, cache)??.into_interpreter();
         Ok(Self(Arc::new(PythonEnvironmentShared {
             root: interpreter.prefix().to_path_buf(),
             interpreter,
@@ -200,18 +214,15 @@ impl PythonEnvironment {
     pub fn lock(&self) -> Result<LockedFile, std::io::Error> {
         if let Some(target) = self.0.interpreter.target() {
             // If we're installing into a `--target`, use a target-specific lock file.
-            LockedFile::acquire(
-                target.root().join(".lock"),
-                target.root().simplified_display(),
-            )
+            LockedFile::acquire(target.root().join(".lock"), target.root().user_display())
         } else if self.0.interpreter.is_virtualenv() {
             // If the environment a virtualenv, use a virtualenv-specific lock file.
-            LockedFile::acquire(self.0.root.join(".lock"), self.0.root.simplified_display())
+            LockedFile::acquire(self.0.root.join(".lock"), self.0.root.user_display())
         } else {
             // Otherwise, use a global lock file.
             LockedFile::acquire(
                 env::temp_dir().join(format!("uv-{}.lock", cache_key::digest(&self.0.root))),
-                self.0.root.simplified_display(),
+                self.0.root.user_display(),
             )
         }
     }
