@@ -1,5 +1,4 @@
 pub use dependency_mode::DependencyMode;
-pub use editables::BuiltEditableMetadata;
 pub use error::ResolveError;
 pub use exclude_newer::ExcludeNewer;
 pub use exclusions::Exclusions;
@@ -25,7 +24,6 @@ mod candidate_selector;
 
 mod dependency_mode;
 mod dependency_provider;
-mod editables;
 mod error;
 mod exclude_newer;
 mod exclusions;
