@@ -4,22 +4,23 @@ use std::collections::{BTreeMap, BTreeSet};
 use std::ops::Bound;
 
 use derivative::Derivative;
-use distribution_types::IndexLocations;
 use indexmap::{IndexMap, IndexSet};
 use owo_colors::OwoColorize;
-use pep440_rs::Version;
 use pubgrub::range::Range;
 use pubgrub::report::{DerivationTree, Derived, External, ReportFormatter};
 use pubgrub::term::Term;
 use pubgrub::type_aliases::Map;
 use rustc_hash::FxHashMap;
+
+use distribution_types::IndexLocations;
+use pep440_rs::{Version, VersionSpecifiers};
 use uv_normalize::PackageName;
 
 use crate::candidate_selector::CandidateSelector;
-use crate::python_requirement::PythonRequirement;
+use crate::python_requirement::{PythonRequirement, RequiresPython};
 use crate::resolver::{IncompletePackage, UnavailablePackage, UnavailableReason};
 
-use super::{PubGrubPackage, PubGrubPackageInner};
+use super::{PubGrubPackage, PubGrubPackageInner, PubGrubPython};
 
 #[derive(Debug)]
 pub(crate) struct PubGrubReportFormatter<'a> {
@@ -44,44 +45,35 @@ impl ReportFormatter<PubGrubPackage, Range<Version>, UnavailableReason>
                 format!("we are solving dependencies of {package} {version}")
             }
             External::NoVersions(package, set) => {
-                if matches!(&**package, PubGrubPackageInner::Python(_)) {
-                    if let Some(python) = self.python_requirement {
-                        if python.target() == python.installed() {
-                            // Simple case, the installed version is the same as the target version
-                            return format!(
-                                "the current {package} version ({}) does not satisfy {}",
-                                python.target(),
+                if let Some(python) = self.python_requirement {
+                    if matches!(
+                        &**package,
+                        PubGrubPackageInner::Python(PubGrubPython::Target)
+                    ) {
+                        return if let Some(target) = python.target() {
+                            format!(
+                                "the requested {package} version ({target}) does not satisfy {}",
                                 PackageRange::compatibility(package, set)
-                            );
-                        }
-                        // Complex case, the target was provided and differs from the installed one
-                        // Determine which Python version requirement was not met
-                        if !set.contains(python.target()) {
-                            return format!(
-                                "the requested {package} version ({}) does not satisfy {}",
-                                python.target(),
+                            )
+                        } else {
+                            format!(
+                                "the requested {package} version does not satisfy {}",
                                 PackageRange::compatibility(package, set)
-                            );
-                        }
-                        // TODO(zanieb): Explain to the user why the installed version is relevant
-                        //               when they provided a target version; probably via a "hint"
-                        debug_assert!(
-                            !set.contains(python.installed()),
-                            "There should not be an incompatibility where the range is satisfied by both Python requirements"
-                        );
+                            )
+                        };
+                    }
+                    if matches!(
+                        &**package,
+                        PubGrubPackageInner::Python(PubGrubPython::Installed)
+                    ) {
                         return format!(
                             "the current {package} version ({}) does not satisfy {}",
                             python.installed(),
                             PackageRange::compatibility(package, set)
                         );
                     }
-                    // We should always have the required Python versions, if we don't we'll fall back
-                    // to a less helpful message in production
-                    debug_assert!(
-                        false,
-                        "Error reporting should always be provided with Python versions"
-                    );
                 }
+
                 let set = self.simplify_set(set, package);
 
                 if set.as_ref() == &Range::full() {
@@ -203,7 +195,15 @@ impl ReportFormatter<PubGrubPackage, Range<Version>, UnavailableReason>
                         _ => (),
                     }
                 }
-                result.push_str(" are incompatible");
+                if let [(p, t)] = slice {
+                    if PackageTerm::new(p, t).plural() {
+                        result.push_str(" are incompatible");
+                    } else {
+                        result.push_str(" is incompatible");
+                    }
+                } else {
+                    result.push_str(" are incompatible");
+                }
                 result
             }
         }
@@ -535,8 +535,27 @@ impl PubGrubReportFormatter<'_> {
                         }
                     }
                 }
+                External::FromDependencyOf(package, package_set, dependency, dependency_set) => {
+                    // Check for no versions due to `Requires-Python`.
+                    if matches!(
+                        &**dependency,
+                        PubGrubPackageInner::Python(PubGrubPython::Target)
+                    ) {
+                        if let Some(python) = self.python_requirement {
+                            if let Some(RequiresPython::Specifiers(specifiers)) = python.target() {
+                                hints.insert(PubGrubHint::RequiresPython {
+                                    requires_python: specifiers.clone(),
+                                    package: package.clone(),
+                                    package_set: self
+                                        .simplify_set(package_set, package)
+                                        .into_owned(),
+                                    package_requires_python: dependency_set.clone(),
+                                });
+                            }
+                        }
+                    }
+                }
                 External::NotRoot(..) => {}
-                External::FromDependencyOf(..) => {}
             },
             DerivationTree::Derived(derived) => {
                 hints.extend(self.hints(
@@ -619,6 +638,16 @@ pub(crate) enum PubGrubHint {
         #[derivative(PartialEq = "ignore", Hash = "ignore")]
         reason: String,
     },
+    /// The `Requires-Python` requirement was not satisfied.
+    RequiresPython {
+        requires_python: VersionSpecifiers,
+        #[derivative(PartialEq = "ignore", Hash = "ignore")]
+        package: PubGrubPackage,
+        #[derivative(PartialEq = "ignore", Hash = "ignore")]
+        package_set: Range<Version>,
+        #[derivative(PartialEq = "ignore", Hash = "ignore")]
+        package_requires_python: Range<Version>,
+    },
 }
 
 impl std::fmt::Display for PubGrubHint {
@@ -710,7 +739,7 @@ impl std::fmt::Display for PubGrubHint {
                     textwrap::indent(reason, "  ")
                 )
             }
-            PubGrubHint::InconsistentVersionMetadata {
+            Self::InconsistentVersionMetadata {
                 package,
                 version,
                 reason,
@@ -725,6 +754,23 @@ impl std::fmt::Display for PubGrubHint {
                     textwrap::indent(reason, "  ")
                 )
             }
+            Self::RequiresPython {
+                requires_python,
+                package,
+                package_set,
+                package_requires_python,
+            } => {
+                write!(
+                    f,
+                    "{}{} The `Requires-Python` requirement ({}) defined in your `pyproject.toml` includes Python versions that are not supported by your dependencies (e.g., {} only supports {}). Consider using a more restrictive `Requires-Python` requirement (like {}).",
+                    "hint".bold().cyan(),
+                    ":".bold(),
+                    requires_python.bold(),
+                    PackageRange::compatibility(package, package_set).bold(),
+                    package_requires_python.bold(),
+                    package_requires_python.bold(),
+                )
+            }
         }
     }
 }
@@ -758,13 +804,28 @@ impl std::fmt::Display for PackageTerm<'_> {
 }
 
 impl PackageTerm<'_> {
+    /// Create a new [`PackageTerm`] from a [`PubGrubPackage`] and a [`Term`].
     fn new<'a>(package: &'a PubGrubPackage, term: &'a Term<Range<Version>>) -> PackageTerm<'a> {
         PackageTerm { package, term }
     }
+
+    /// Returns `true` if the predicate following this package term should be singular or plural.
+    fn plural(&self) -> bool {
+        match self.term {
+            Term::Positive(set) => PackageRange::compatibility(self.package, set).plural(),
+            Term::Negative(set) => {
+                if set.as_singleton().is_some() {
+                    false
+                } else {
+                    PackageRange::compatibility(self.package, &set.complement()).plural()
+                }
+            }
+        }
+    }
 }
 
 /// The kind of version ranges being displayed in [`PackageRange`]
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 enum PackageRangeKind {
     Dependency,
     Compatibility,
@@ -784,12 +845,19 @@ impl PackageRange<'_> {
     /// be singular or plural e.g. if false use "<range> depends on <...>" and
     /// if true use "<range> depend on <...>"
     fn plural(&self) -> bool {
-        if self.range.is_empty() {
-            false
+        let mut segments = self.range.iter();
+        if let Some(segment) = segments.next() {
+            // A single unbounded compatibility segment is always plural ("all versions of").
+            if self.kind == PackageRangeKind::Compatibility {
+                if matches!(segment, (Bound::Unbounded, Bound::Unbounded)) {
+                    return true;
+                }
+            }
+            // Otherwise, multiple segments are always plural.
+            segments.next().is_some()
         } else {
-            let segments: Vec<_> = self.range.iter().collect();
-            // "all versions of" is the only plural case
-            matches!(segments.as_slice(), [(Bound::Unbounded, Bound::Unbounded)])
+            // An empty range is always singular.
+            false
         }
     }
 }
