@@ -372,8 +372,6 @@ pub(crate) struct PipCompileArgs {
     #[arg(long, env = "UV_CUSTOM_COMPILE_COMMAND")]
     pub(crate) custom_compile_command: Option<String>,
 
-    /// Run offline, i.e., without accessing the network.
-
     /// Refresh all cached data.
     #[arg(long, conflicts_with("offline"), overrides_with("no_refresh"))]
     pub(crate) refresh: bool,
@@ -399,42 +397,8 @@ pub(crate) struct PipCompileArgs {
     #[arg(long, value_enum, env = "UV_LINK_MODE")]
     pub(crate) link_mode: Option<install_wheel_rs::linker::LinkMode>,
 
-    /// The URL of the Python package index (by default: <https://pypi.org/simple>).
-    ///
-    /// The index given by this flag is given lower priority than all other
-    /// indexes specified via the `--extra-index-url` flag.
-    ///
-    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
-    /// as it finds it in an index. That is, it isn't possible for `uv` to
-    /// consider versions of the same package across multiple indexes.
-    #[arg(long, short, env = "UV_INDEX_URL", value_parser = parse_index_url)]
-    pub(crate) index_url: Option<Maybe<IndexUrl>>,
-
-    /// Extra URLs of package indexes to use, in addition to `--index-url`.
-    ///
-    /// All indexes given via this flag take priority over the index
-    /// in `--index-url` (which defaults to PyPI). And when multiple
-    /// `--extra-index-url` flags are given, earlier values take priority.
-    ///
-    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
-    /// as it finds it in an index. That is, it isn't possible for `uv` to
-    /// consider versions of the same package across multiple indexes.
-    #[arg(long, env = "UV_EXTRA_INDEX_URL", value_delimiter = ' ', value_parser = parse_index_url)]
-    pub(crate) extra_index_url: Option<Vec<Maybe<IndexUrl>>>,
-
-    /// Locations to search for candidate distributions, beyond those found in the indexes.
-    ///
-    /// If a path, the target must be a directory that contains package as wheel files (`.whl`) or
-    /// source distributions (`.tar.gz` or `.zip`) at the top level.
-    ///
-    /// If a URL, the page must contain a flat list of links to package files.
-    #[arg(long, short)]
-    pub(crate) find_links: Option<Vec<FlatIndexLocation>>,
-
-    /// Ignore the registry index (e.g., PyPI), instead relying on direct URL dependencies and those
-    /// discovered via `--find-links`.
-    #[arg(long)]
-    pub(crate) no_index: bool,
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
 
     /// The strategy to use when resolving against multiple index URLs.
     ///
@@ -465,7 +429,7 @@ pub(crate) struct PipCompileArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, verbatim_doc_comment, group = "discovery")]
+    #[arg(long, verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// Install packages into the system Python.
@@ -478,7 +442,6 @@ pub(crate) struct PipCompileArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -563,8 +526,8 @@ pub(crate) struct PipCompileArgs {
     /// The minimum Python version that should be supported by the compiled requirements (e.g.,
     /// `3.7` or `3.7.9`).
     ///
-    /// If a patch version is omitted, the most recent known patch version for that minor version
-    /// is assumed. For example, `3.7` is mapped to `3.7.17`.
+    /// If a patch version is omitted, the minimum patch version is assumed. For example, `3.7` is
+    /// mapped to `3.7.0`.
     #[arg(long, short)]
     pub(crate) python_version: Option<PythonVersion>,
 
@@ -622,12 +585,6 @@ pub(crate) struct PipCompileArgs {
     #[arg(long, overrides_with("emit_index_annotation"), hide = true)]
     pub(crate) no_emit_index_annotation: bool,
 
-    #[arg(long, overrides_with("no_unstable_uv_lock_file"), hide = true)]
-    pub(crate) unstable_uv_lock_file: bool,
-
-    #[arg(long, overrides_with("unstable_uv_lock_file"), hide = true)]
-    pub(crate) no_unstable_uv_lock_file: bool,
-
     #[command(flatten)]
     pub(crate) compat_args: compat::PipCompileCompatArgs,
 }
@@ -683,42 +640,8 @@ pub(crate) struct PipSyncArgs {
     #[arg(long, value_enum, env = "UV_LINK_MODE")]
     pub(crate) link_mode: Option<install_wheel_rs::linker::LinkMode>,
 
-    /// The URL of the Python package index (by default: <https://pypi.org/simple>).
-    ///
-    /// The index given by this flag is given lower priority than all other
-    /// indexes specified via the `--extra-index-url` flag.
-    ///
-    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
-    /// as it finds it in an index. That is, it isn't possible for `uv` to
-    /// consider versions of the same package across multiple indexes.
-    #[arg(long, short, env = "UV_INDEX_URL", value_parser = parse_index_url)]
-    pub(crate) index_url: Option<Maybe<IndexUrl>>,
-
-    /// Extra URLs of package indexes to use, in addition to `--index-url`.
-    ///
-    /// All indexes given via this flag take priority over the index
-    /// in `--index-url` (which defaults to PyPI). And when multiple
-    /// `--extra-index-url` flags are given, earlier values take priority.
-    ///
-    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
-    /// as it finds it in an index. That is, it isn't possible for `uv` to
-    /// consider versions of the same package across multiple indexes.
-    #[arg(long, env = "UV_EXTRA_INDEX_URL", value_delimiter = ' ', value_parser = parse_index_url)]
-    pub(crate) extra_index_url: Option<Vec<Maybe<IndexUrl>>>,
-
-    /// Locations to search for candidate distributions, beyond those found in the indexes.
-    ///
-    /// If a path, the target must be a directory that contains package as wheel files (`.whl`) or
-    /// source distributions (`.tar.gz` or `.zip`) at the top level.
-    ///
-    /// If a URL, the page must contain a flat list of links to package files.
-    #[arg(long, short)]
-    pub(crate) find_links: Option<Vec<FlatIndexLocation>>,
-
-    /// Ignore the registry index (e.g., PyPI), instead relying on direct URL dependencies and those
-    /// discovered via `--find-links`.
-    #[arg(long)]
-    pub(crate) no_index: bool,
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
 
     /// The strategy to use when resolving against multiple index URLs.
     ///
@@ -768,13 +691,7 @@ pub(crate) struct PipSyncArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// Install packages into the system Python.
@@ -789,7 +706,6 @@ pub(crate) struct PipSyncArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -807,7 +723,6 @@ pub(crate) struct PipSyncArgs {
         long,
         env = "UV_BREAK_SYSTEM_PACKAGES",
         value_parser = clap::builder::BoolishValueParser::new(),
-        requires = "discovery",
         overrides_with("no_break_system_packages")
     )]
     pub(crate) break_system_packages: bool,
@@ -831,7 +746,6 @@ pub(crate) struct PipSyncArgs {
     /// Disable isolation when building source distributions.
     ///
     /// Assumes that build dependencies specified by PEP 518 are already installed.
-
     #[arg(
         long,
         env = "UV_NO_BUILD_ISOLATION",
@@ -915,8 +829,8 @@ pub(crate) struct PipSyncArgs {
     /// The minimum Python version that should be supported by the requirements (e.g.,
     /// `3.7` or `3.7.9`).
     ///
-    /// If a patch version is omitted, the most recent known patch version for that minor version
-    /// is assumed. For example, `3.7` is mapped to `3.7.17`.
+    /// If a patch version is omitted, the minimum patch version is assumed. For example, `3.7` is
+    /// mapped to `3.7.0`.
     #[arg(long)]
     pub(crate) python_version: Option<PythonVersion>,
 
@@ -1078,42 +992,8 @@ pub(crate) struct PipInstallArgs {
     #[arg(long, hide = true)]
     pub(crate) pre: bool,
 
-    /// The URL of the Python package index (by default: <https://pypi.org/simple>).
-    ///
-    /// The index given by this flag is given lower priority than all other
-    /// indexes specified via the `--extra-index-url` flag.
-    ///
-    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
-    /// as it finds it in an index. That is, it isn't possible for `uv` to
-    /// consider versions of the same package across multiple indexes.
-    #[arg(long, short, env = "UV_INDEX_URL", value_parser = parse_index_url)]
-    pub(crate) index_url: Option<Maybe<IndexUrl>>,
-
-    /// Extra URLs of package indexes to use, in addition to `--index-url`.
-    ///
-    /// All indexes given via this flag take priority over the index
-    /// in `--index-url` (which defaults to PyPI). And when multiple
-    /// `--extra-index-url` flags are given, earlier values take priority.
-    ///
-    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
-    /// as it finds it in an index. That is, it isn't possible for `uv` to
-    /// consider versions of the same package across multiple indexes.
-    #[arg(long, env = "UV_EXTRA_INDEX_URL", value_delimiter = ' ', value_parser = parse_index_url)]
-    pub(crate) extra_index_url: Option<Vec<Maybe<IndexUrl>>>,
-
-    /// Locations to search for candidate distributions, beyond those found in the indexes.
-    ///
-    /// If a path, the target must be a directory that contains package as wheel files (`.whl`) or
-    /// source distributions (`.tar.gz` or `.zip`) at the top level.
-    ///
-    /// If a URL, the page must contain a flat list of links to package files.
-    #[arg(long, short)]
-    pub(crate) find_links: Option<Vec<FlatIndexLocation>>,
-
-    /// Ignore the registry index (e.g., PyPI), instead relying on direct URL dependencies and those
-    /// discovered via `--find-links`.
-    #[arg(long)]
-    pub(crate) no_index: bool,
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
 
     /// The strategy to use when resolving against multiple index URLs.
     ///
@@ -1135,8 +1015,12 @@ pub(crate) struct PipInstallArgs {
     /// - Editable installs are not supported.
     /// - Local dependencies are not supported, unless they point to a specific wheel (`.whl`) or
     ///   source archive (`.zip`, `.tar.gz`), as opposed to a directory.
-    #[arg(long,         env = "UV_REQUIRE_HASHES",
-        value_parser = clap::builder::BoolishValueParser::new(), overrides_with("no_require_hashes"))]
+    #[arg(
+        long,
+        env = "UV_REQUIRE_HASHES",
+        value_parser = clap::builder::BoolishValueParser::new(),
+        overrides_with("no_require_hashes"),
+    )]
     pub(crate) require_hashes: bool,
 
     #[arg(long, overrides_with("require_hashes"), hide = true)]
@@ -1163,13 +1047,7 @@ pub(crate) struct PipInstallArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// Install packages into the system Python.
@@ -1184,7 +1062,6 @@ pub(crate) struct PipInstallArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1202,7 +1079,6 @@ pub(crate) struct PipInstallArgs {
         long,
         env = "UV_BREAK_SYSTEM_PACKAGES",
         value_parser = clap::builder::BoolishValueParser::new(),
-        requires = "discovery",
         overrides_with("no_break_system_packages")
     )]
     pub(crate) break_system_packages: bool,
@@ -1309,8 +1185,8 @@ pub(crate) struct PipInstallArgs {
     /// The minimum Python version that should be supported by the requirements (e.g.,
     /// `3.7` or `3.7.9`).
     ///
-    /// If a patch version is omitted, the most recent known patch version for that minor version
-    /// is assumed. For example, `3.7` is mapped to `3.7.17`.
+    /// If a patch version is omitted, the minimum patch version is assumed. For example, `3.7` is
+    /// mapped to `3.7.0`.
     #[arg(long)]
     pub(crate) python_version: Option<PythonVersion>,
 
@@ -1348,9 +1224,6 @@ pub(crate) struct PipInstallArgs {
     #[arg(long)]
     pub(crate) dry_run: bool,
 
-    #[arg(long, hide = true, group = "sources")]
-    pub(crate) unstable_uv_lock_file: Option<String>,
-
     #[command(flatten)]
     pub(crate) compat_args: compat::PipInstallCompatArgs,
 }
@@ -1379,13 +1252,7 @@ pub(crate) struct PipUninstallArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// Attempt to use `keyring` for authentication for remote requirements files.
@@ -1409,7 +1276,6 @@ pub(crate) struct PipUninstallArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1427,7 +1293,6 @@ pub(crate) struct PipUninstallArgs {
         long,
         env = "UV_BREAK_SYSTEM_PACKAGES",
         value_parser = clap::builder::BoolishValueParser::new(),
-        requires = "discovery",
         overrides_with("no_break_system_packages")
     )]
     pub(crate) break_system_packages: bool,
@@ -1467,13 +1332,7 @@ pub(crate) struct PipFreezeArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1489,7 +1348,6 @@ pub(crate) struct PipFreezeArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1536,13 +1394,7 @@ pub(crate) struct PipListArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1558,7 +1410,6 @@ pub(crate) struct PipListArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1584,13 +1435,7 @@ pub(crate) struct PipCheckArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1606,7 +1451,6 @@ pub(crate) struct PipCheckArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1640,13 +1484,7 @@ pub(crate) struct PipShowArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1662,7 +1500,6 @@ pub(crate) struct PipShowArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1684,13 +1521,7 @@ pub(crate) struct VenvArgs {
     ///
     /// Note that this is different from `--python-version` in `pip compile`, which takes `3.10` or `3.10.13` and
     /// doesn't look for a Python interpreter on disk.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 
     /// Use the system Python to uninstall packages.
@@ -1705,7 +1536,6 @@ pub(crate) struct VenvArgs {
         long,
         env = "UV_SYSTEM_PYTHON",
         value_parser = clap::builder::BoolishValueParser::new(),
-        group = "discovery",
         overrides_with("no_system")
     )]
     pub(crate) system: bool,
@@ -1824,6 +1654,17 @@ pub(crate) struct VenvArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub(crate) struct RunArgs {
+    /// Include optional dependencies in the given extra group name; may be provided more than once.
+    #[arg(long, conflicts_with = "all_extras", value_parser = extra_name_with_clap_error)]
+    pub(crate) extra: Option<Vec<ExtraName>>,
+
+    /// Include all optional dependencies.
+    #[arg(long, conflicts_with = "extra")]
+    pub(crate) all_extras: bool,
+
+    #[arg(long, overrides_with("all_extras"), hide = true)]
+    pub(crate) no_all_extras: bool,
+
     /// The command to run.
     pub(crate) target: Option<String>,
 
@@ -1835,6 +1676,36 @@ pub(crate) struct RunArgs {
     #[arg(long)]
     pub(crate) with: Vec<String>,
 
+    /// Refresh all cached data.
+    #[arg(long, conflicts_with("offline"), overrides_with("no_refresh"))]
+    pub(crate) refresh: bool,
+
+    #[arg(
+        long,
+        conflicts_with("offline"),
+        overrides_with("refresh"),
+        hide = true
+    )]
+    pub(crate) no_refresh: bool,
+
+    /// Refresh cached data for a specific package.
+    #[arg(long)]
+    pub(crate) refresh_package: Vec<PackageName>,
+
+    /// Allow package upgrades, ignoring pinned versions in the existing lockfile.
+    #[arg(long, short = 'U', overrides_with("no_upgrade"))]
+    pub(crate) upgrade: bool,
+
+    #[arg(long, overrides_with("upgrade"), hide = true)]
+    pub(crate) no_upgrade: bool,
+
+    /// Allow upgrades for a specific package, ignoring pinned versions in the existing lockfile.
+    #[arg(long, short = 'P')]
+    pub(crate) upgrade_package: Vec<PackageName>,
+
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
+
     /// The Python interpreter to use to build the run environment.
     ///
     /// By default, `uv` uses the virtual environment in the current working directory or any parent
@@ -1846,19 +1717,54 @@ pub(crate) struct RunArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
+
+    /// Limit candidate packages to those that were uploaded prior to the given date.
+    ///
+    /// Accepts both RFC 3339 timestamps (e.g., `2006-12-02T02:07:43Z`) and UTC dates in the same
+    /// format (e.g., `2006-12-02`).
+    #[arg(long)]
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
+
+    /// Run the command in a different package in the workspace.
+    #[arg(long, conflicts_with = "isolated")]
+    pub(crate) package: Option<PackageName>,
 }
 
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub(crate) struct SyncArgs {
+    /// Include optional dependencies in the given extra group name; may be provided more than once.
+    #[arg(long, conflicts_with = "all_extras", value_parser = extra_name_with_clap_error)]
+    pub(crate) extra: Option<Vec<ExtraName>>,
+
+    /// Include all optional dependencies.
+    #[arg(long, conflicts_with = "extra")]
+    pub(crate) all_extras: bool,
+
+    #[arg(long, overrides_with("all_extras"), hide = true)]
+    pub(crate) no_all_extras: bool,
+
+    /// Refresh all cached data.
+    #[arg(long, conflicts_with("offline"), overrides_with("no_refresh"))]
+    pub(crate) refresh: bool,
+
+    #[arg(
+        long,
+        conflicts_with("offline"),
+        overrides_with("refresh"),
+        hide = true
+    )]
+    pub(crate) no_refresh: bool,
+
+    /// Refresh cached data for a specific package.
+    #[arg(long)]
+    pub(crate) refresh_package: Vec<PackageName>,
+
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
+
     /// The Python interpreter to use to build the run environment.
     ///
     /// By default, `uv` uses the virtual environment in the current working directory or any parent
@@ -1870,19 +1776,43 @@ pub(crate) struct SyncArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 }
 
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub(crate) struct LockArgs {
+    /// Refresh all cached data.
+    #[arg(long, conflicts_with("offline"), overrides_with("no_refresh"))]
+    pub(crate) refresh: bool,
+
+    #[arg(
+        long,
+        conflicts_with("offline"),
+        overrides_with("refresh"),
+        hide = true
+    )]
+    pub(crate) no_refresh: bool,
+
+    /// Refresh cached data for a specific package.
+    #[arg(long)]
+    pub(crate) refresh_package: Vec<PackageName>,
+
+    /// Allow package upgrades, ignoring pinned versions in the existing lockfile.
+    #[arg(long, short = 'U', overrides_with("no_upgrade"))]
+    pub(crate) upgrade: bool,
+
+    #[arg(long, overrides_with("upgrade"), hide = true)]
+    pub(crate) no_upgrade: bool,
+
+    /// Allow upgrades for a specific package, ignoring pinned versions in the existing lockfile.
+    #[arg(long, short = 'P')]
+    pub(crate) upgrade_package: Vec<PackageName>,
+
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
+
     /// The Python interpreter to use to build the run environment.
     ///
     /// By default, `uv` uses the virtual environment in the current working directory or any parent
@@ -1894,14 +1824,15 @@ pub(crate) struct LockArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
+
+    /// Limit candidate packages to those that were uploaded prior to the given date.
+    ///
+    /// Accepts both RFC 3339 timestamps (e.g., `2006-12-02T02:07:43Z`) and UTC dates in the same
+    /// format (e.g., `2006-12-02`).
+    #[arg(long)]
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
 }
 
 #[derive(Args)]
@@ -1946,13 +1877,64 @@ pub(crate) struct ToolRunArgs {
     #[arg(long)]
     pub(crate) from: Option<String>,
 
+    /// Include the following extra requirements.
+    #[arg(long)]
+    pub(crate) with: Vec<String>,
+
+    #[command(flatten)]
+    pub(crate) index_args: IndexArgs,
+
     /// The Python interpreter to use to build the run environment.
-    #[arg(
-        long,
-        short,
-        env = "UV_PYTHON",
-        verbatim_doc_comment,
-        group = "discovery"
-    )]
+    ///
+    /// By default, `uv` uses the virtual environment in the current working directory or any parent
+    /// directory, falling back to searching for a Python executable in `PATH`. The `--python`
+    /// option allows you to specify a different interpreter.
+    ///
+    /// Supported formats:
+    /// - `3.10` looks for an installed Python 3.10 using `py --list-paths` on Windows, or
+    ///   `python3.10` on Linux and macOS.
+    /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
+    /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub(crate) python: Option<String>,
 }
+
+#[derive(Args)]
+pub(crate) struct IndexArgs {
+    /// The URL of the Python package index (by default: <https://pypi.org/simple>).
+    ///
+    /// The index given by this flag is given lower priority than all other
+    /// indexes specified via the `--extra-index-url` flag.
+    ///
+    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
+    /// as it finds it in an index. That is, it isn't possible for `uv` to
+    /// consider versions of the same package across multiple indexes.
+    #[arg(long, short, env = "UV_INDEX_URL", value_parser = parse_index_url)]
+    pub(crate) index_url: Option<Maybe<IndexUrl>>,
+
+    /// Extra URLs of package indexes to use, in addition to `--index-url`.
+    ///
+    /// All indexes given via this flag take priority over the index
+    /// in `--index-url` (which defaults to PyPI). And when multiple
+    /// `--extra-index-url` flags are given, earlier values take priority.
+    ///
+    /// Unlike `pip`, `uv` will stop looking for versions of a package as soon
+    /// as it finds it in an index. That is, it isn't possible for `uv` to
+    /// consider versions of the same package across multiple indexes.
+    #[arg(long, env = "UV_EXTRA_INDEX_URL", value_delimiter = ' ', value_parser = parse_index_url)]
+    pub(crate) extra_index_url: Option<Vec<Maybe<IndexUrl>>>,
+
+    /// Locations to search for candidate distributions, beyond those found in the indexes.
+    ///
+    /// If a path, the target must be a directory that contains package as wheel files (`.whl`) or
+    /// source distributions (`.tar.gz` or `.zip`) at the top level.
+    ///
+    /// If a URL, the page must contain a flat list of links to package files.
+    #[arg(long, short)]
+    pub(crate) find_links: Option<Vec<FlatIndexLocation>>,
+
+    /// Ignore the registry index (e.g., PyPI), instead relying on direct URL dependencies and those
+    /// discovered via `--find-links`.
+    #[arg(long)]
+    pub(crate) no_index: bool,
+}
