@@ -1,8 +1,10 @@
 use anstream::eprint;
 use anyhow::Result;
+use std::borrow::Cow;
 
 use distribution_types::{IndexLocations, UnresolvedRequirementSpecification};
 use install_wheel_rs::linker::LinkMode;
+use pep440_rs::{VersionSpecifier, VersionSpecifiers};
 use uv_cache::Cache;
 use uv_client::RegistryClientBuilder;
 use uv_configuration::{
@@ -91,8 +93,25 @@ pub(super) async fn do_lock(
     let source_trees = vec![];
     let project_name = project.project_name().clone();
 
+    // Determine the supported Python range. If no range is defined, and warn and default to the
+    // current minor version.
+    let requires_python = if let Some(requires_python) =
+        project.current_project().project().requires_python.as_ref()
+    {
+        Cow::Borrowed(requires_python)
+    } else {
+        let requires_python = VersionSpecifiers::from(
+            VersionSpecifier::greater_than_equal_version(venv.interpreter().python_minor_version()),
+        );
+        warn_user!(
+            "No `requires-python` field found in `{}`. Defaulting to `{requires_python}`.",
+            project.current_project().project().name,
+        );
+        Cow::Owned(requires_python)
+    };
+
     // Determine the tags, markers, and interpreter to use for resolution.
-    let interpreter = venv.interpreter().clone();
+    let interpreter = venv.interpreter();
     let tags = venv.interpreter().tags()?;
     let markers = venv.interpreter().markers();
 
@@ -131,7 +150,7 @@ pub(super) async fn do_lock(
     let build_dispatch = BuildDispatch::new(
         &client,
         cache,
-        &interpreter,
+        interpreter,
         index_locations,
         &flat_index,
         &index,
@@ -160,9 +179,10 @@ pub(super) async fn do_lock(
         &hasher,
         &reinstall,
         &upgrade,
-        &interpreter,
+        interpreter,
         tags,
         None,
+        Some(&requires_python),
         &client,
         &flat_index,
         &index,
@@ -178,7 +198,7 @@ pub(super) async fn do_lock(
     pip::operations::diagnose_resolution(resolution.diagnostics(), printer)?;
 
     // Write the lockfile to disk.
-    let lock = resolution.lock()?;
+    let lock = Lock::from_resolution_graph(&resolution)?;
     let encoded = lock.to_toml()?;
     fs_err::tokio::write(
         project.workspace().root().join("uv.lock"),
