@@ -7,6 +7,7 @@ use tracing::debug;
 
 use distribution_types::{IndexLocations, Resolution};
 use install_wheel_rs::linker::LinkMode;
+use pep440_rs::{Version, VersionSpecifiers};
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, RegistryClientBuilder};
 use uv_configuration::{
@@ -32,6 +33,9 @@ pub(crate) mod sync;
 
 #[derive(thiserror::Error, Debug)]
 pub(crate) enum ProjectError {
+    #[error("The current Python version ({0}) is not compatible with the locked Python requirement ({1})")]
+    RequiresPython(Version, VersionSpecifiers),
+
     #[error(transparent)]
     Interpreter(#[from] uv_interpreter::Error),
 
@@ -149,7 +153,7 @@ pub(crate) async fn update_environment(
     }
 
     // Determine the tags, markers, and interpreter to use for resolution.
-    let interpreter = venv.interpreter().clone();
+    let interpreter = venv.interpreter();
     let tags = venv.interpreter().tags()?;
     let markers = venv.interpreter().markers();
 
@@ -187,7 +191,7 @@ pub(crate) async fn update_environment(
     let resolve_dispatch = BuildDispatch::new(
         &client,
         cache,
-        &interpreter,
+        interpreter,
         index_locations,
         &flat_index,
         &index,
@@ -216,9 +220,10 @@ pub(crate) async fn update_environment(
         &hasher,
         &reinstall,
         &upgrade,
-        &interpreter,
+        interpreter,
         tags,
         Some(markers),
+        None,
         &client,
         &flat_index,
         &index,
@@ -245,7 +250,7 @@ pub(crate) async fn update_environment(
         BuildDispatch::new(
             &client,
             cache,
-            &interpreter,
+            interpreter,
             index_locations,
             &flat_index,
             &index,
