@@ -2,6 +2,7 @@ use std::ffi::OsString;
 use std::path::PathBuf;
 
 use anyhow::{Context, Result};
+use distribution_types::IndexLocations;
 use itertools::Itertools;
 use tempfile::tempdir_in;
 use tokio::process::Command;
@@ -28,6 +29,7 @@ pub(crate) async fn run(
     with: Vec<String>,
     _isolated: bool,
     preview: PreviewMode,
+    index_locations: IndexLocations,
     connectivity: Connectivity,
     cache: &Cache,
     printer: Printer,
@@ -75,11 +77,11 @@ pub(crate) async fn run(
         update_environment(
             venv,
             &requirements,
-            None,
-            preview,
+            &index_locations,
             connectivity,
             cache,
             printer,
+            preview,
         )
         .await?,
     );
