@@ -107,6 +107,11 @@ async fn run() -> Result<ExitStatus> {
         }
     };
 
+    // enable flag to pick up warnings generated by workspace loading.
+    if !cli.global_args.quiet {
+        uv_warnings::enable();
+    }
+
     // Load the workspace settings, prioritizing (in order):
     // 1. The configuration file specified on the command-line.
     // 2. The configuration file in the current directory.
@@ -148,7 +153,9 @@ async fn run() -> Result<ExitStatus> {
     };
 
     // Configure the `warn!` macros, which control user-facing warnings in the CLI.
-    if !globals.quiet {
+    if globals.quiet {
+        uv_warnings::disable();
+    } else {
         uv_warnings::enable();
     }
 
