@@ -1,7 +1,10 @@
 use std::env;
 use std::path::PathBuf;
+use std::process::Command;
 
-use crate::common::{get_bin, uv_snapshot, TestContext, EXCLUDE_NEWER};
+use anyhow::Result;
+
+use crate::common::{copy_dir_ignore, get_bin, uv_snapshot, TestContext, EXCLUDE_NEWER};
 
 mod common;
 
@@ -10,8 +13,8 @@ mod common;
 /// The goal of the workspace tests is to resolve local workspace packages correctly. We add some
 /// non-workspace dependencies to ensure that transitive non-workspace dependencies are also
 /// correctly resolved.
-pub fn install_workspace(context: &TestContext) -> std::process::Command {
-    let mut command = std::process::Command::new(get_bin());
+fn install_workspace(context: &TestContext) -> Command {
+    let mut command = Command::new(get_bin());
     command
         .arg("pip")
         .arg("install")
@@ -34,6 +37,28 @@ pub fn install_workspace(context: &TestContext) -> std::process::Command {
     command
 }
 
+/// A `uv run` command.
+fn run_workspace(context: &TestContext) -> Command {
+    let mut command = Command::new(get_bin());
+    command
+        .arg("run")
+        .arg("--preview")
+        .arg("--cache-dir")
+        .arg(context.cache_dir.path())
+        .arg("--python")
+        .arg(context.interpreter())
+        .arg("--exclude-newer")
+        .arg(EXCLUDE_NEWER)
+        .env("UV_NO_WRAP", "1");
+
+    if cfg!(all(windows, debug_assertions)) {
+        // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
+        // default windows stack of 1MB
+        command.env("UV_STACK_SIZE", (4 * 1024 * 1024).to_string());
+    }
+    command
+}
+
 fn workspaces_dir() -> PathBuf {
     env::current_dir()
         .unwrap()
@@ -218,7 +243,7 @@ fn test_albatross_root_workspace() {
     ----- stdout -----
 
     ----- stderr -----
-    Audited 3 packages in [TIME]
+    Audited 1 package in [TIME]
     "###
     );
 
@@ -257,7 +282,7 @@ fn test_albatross_root_workspace_bird_feeder() {
     ----- stdout -----
 
     ----- stderr -----
-    Audited 2 packages in [TIME]
+    Audited 1 package in [TIME]
     "###
     );
 
@@ -296,7 +321,7 @@ fn test_albatross_root_workspace_albatross() {
     ----- stdout -----
 
     ----- stderr -----
-    Audited 2 packages in [TIME]
+    Audited 1 package in [TIME]
     "###
     );
 
@@ -335,9 +360,148 @@ fn test_albatross_virtual_workspace() {
     ----- stdout -----
 
     ----- stderr -----
-    Audited 2 packages in [TIME]
+    Audited 1 package in [TIME]
     "###
     );
 
     context.assert_file(current_dir.join("check_installed_bird_feeder.py"));
 }
+
+/// Check that `uv run --package` works in a virtual workspace.
+#[test]
+fn test_uv_run_with_package_virtual_workspace() -> Result<()> {
+    let context = TestContext::new("3.12");
+    let work_dir = context.temp_dir.join("albatross-virtual-workspace");
+
+    copy_dir_ignore(
+        workspaces_dir().join("albatross-virtual-workspace"),
+        &work_dir,
+    )?;
+
+    // TODO(konsti): `--python` is being ignored atm, so we need to create the correct venv
+    // ourselves and add the output filters.
+    let venv = work_dir.join(".venv");
+    assert_cmd::Command::new(get_bin())
+        .arg("venv")
+        .arg("-p")
+        .arg(context.interpreter())
+        .arg(&venv)
+        .assert();
+
+    let mut filters = context.filters();
+    filters.push((
+        r"Using Python 3.12.\[X\] interpreter at: .*",
+        "Using Python 3.12.[X] interpreter at: [PYTHON]",
+    ));
+
+    uv_snapshot!(filters, run_workspace(&context)
+        .arg("--package")
+        .arg("bird-feeder")
+        .arg("packages/bird-feeder/check_installed_bird_feeder.py")
+        .current_dir(&work_dir), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Success
+
+    ----- stderr -----
+    Resolved 7 packages in [TIME]
+    Downloaded 5 packages in [TIME]
+    Installed 5 packages in [TIME]
+     + anyio==4.3.0
+     + bird-feeder==1.0.0 (from file://[TEMP_DIR]/albatross-virtual-workspace/packages/bird-feeder)
+     + idna==3.6
+     + seeds==1.0.0 (from file://[TEMP_DIR]/albatross-virtual-workspace/packages/seeds)
+     + sniffio==1.3.1
+    "###
+    );
+
+    uv_snapshot!(context.filters(), universal_windows_filters=true, run_workspace(&context)
+            .arg("--package")
+            .arg("albatross")
+            .arg("packages/albatross/check_installed_albatross.py")
+            .current_dir(&work_dir), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Success
+
+    ----- stderr -----
+    Resolved 10 packages in [TIME]
+    Downloaded 2 packages in [TIME]
+    Installed 2 packages in [TIME]
+     + albatross==0.1.0 (from file://[TEMP_DIR]/albatross-virtual-workspace/packages/albatross)
+     + tqdm==4.66.2
+    "###
+    );
+
+    Ok(())
+}
+
+/// Check that `uv run --package` works in a root workspace.
+#[test]
+fn test_uv_run_with_package_root_workspace() -> Result<()> {
+    let context = TestContext::new("3.12");
+    let work_dir = context.temp_dir.join("albatross-root-workspace");
+
+    copy_dir_ignore(workspaces_dir().join("albatross-root-workspace"), &work_dir)?;
+
+    // TODO(konsti): `--python` is being ignored atm, so we need to create the correct venv
+    // ourselves and add the output filters.
+    let venv = work_dir.join(".venv");
+    assert_cmd::Command::new(get_bin())
+        .arg("venv")
+        .arg("-p")
+        .arg(context.interpreter())
+        .arg(&venv)
+        .assert();
+
+    let mut filters = context.filters();
+    filters.push((
+        r"Using Python 3.12.\[X\] interpreter at: .*",
+        "Using Python 3.12.[X] interpreter at: [PYTHON]",
+    ));
+
+    uv_snapshot!(filters, run_workspace(&context)
+        .arg("--package")
+        .arg("bird-feeder")
+        .arg("packages/bird-feeder/check_installed_bird_feeder.py")
+        .current_dir(&work_dir), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Success
+
+    ----- stderr -----
+    Resolved 7 packages in [TIME]
+    Downloaded 5 packages in [TIME]
+    Installed 5 packages in [TIME]
+     + anyio==4.3.0
+     + bird-feeder==1.0.0 (from file://[TEMP_DIR]/albatross-root-workspace/packages/bird-feeder)
+     + idna==3.6
+     + seeds==1.0.0 (from file://[TEMP_DIR]/albatross-root-workspace/packages/seeds)
+     + sniffio==1.3.1
+    "###
+    );
+
+    uv_snapshot!(context.filters(), universal_windows_filters=true, run_workspace(&context)
+            .arg("--package")
+            .arg("albatross")
+            .arg("check_installed_albatross.py")
+            .current_dir(&work_dir), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Success
+
+    ----- stderr -----
+    Resolved 10 packages in [TIME]
+    Downloaded 2 packages in [TIME]
+    Installed 2 packages in [TIME]
+     + albatross==0.1.0 (from file://[TEMP_DIR]/albatross-root-workspace)
+     + tqdm==4.66.2
+    "###
+    );
+
+    Ok(())
+}
